@@ -15,10 +15,10 @@ pub struct InstanceAccess<'i, Id> {
 }
 
 impl<'r, Id> FromFormValue<'r> for InstanceAccess<'r, Id>
-  where Id: AccessId, OE: From<Id::Error>
+  where Id: AccessId, Fatal: From<Id::Error>
 {
-  type Error = OER;
-  #[throws(OER)]
+  type Error = FER;
+  #[throws(FER)]
   fn from_form_value(param: &'r RawStr) -> Self {
     let token = RawTokenVal::from_str(param.as_str());
     let i = InstanceAccessDetails::from_token(token)?;
@@ -32,19 +32,28 @@ struct ApiPiece<O:op::Complex> {
   piece: VisiblePieceId,
   gen: Generation,
   cseq: ClientSequence,
+  #[serde(default)] loose: bool,
   op: O,
 }
 
+#[derive(Debug)]
+pub struct ContinueDespiteConflict;
+
 mod op {
   use super::*;
 
   pub trait Core: Debug { 
-    #[throws(OnlineError)]
+    #[throws(Inapplicable)]
     fn check_held(&self, pc: &GPiece, player: PlayerId) {
       if pc.held != None && pc.held != Some(player) {
-        throw!(OnlineError::PieceHeld)
+        throw!(Ia::PieceHeld)
       }
     }
+
+    fn conflict_loose_check(&self, _gpc: &GPiece, _client: ClientId)
+        -> Result<ContinueDespiteConflict, ApiPieceOpError> {
+      throw!(Fatal::BadLoose)
+    }
   }
 
   pub trait Simple: Debug { 
@@ -67,26 +76,22 @@ mod op {
 
 #[derive(Error,Debug)]
 #[error("{0}")]
-pub struct OnlineErrorResponse(#[from] OnlineError);
+pub struct FatalErrorResponse(#[from] Fatal);
 
-impl From<&OnlineErrorResponse> for rocket::http::Status {
-  fn from(oe: &OnlineErrorResponse) -> rocket::http::Status {
-    use OnlineError::*;
+impl From<&FatalErrorResponse> for rocket::http::Status {
+  fn from(oe: &FatalErrorResponse) -> rocket::http::Status {
+    use Fatal::*;
     match oe.0 {
       ServerFailure(_) => Status::InternalServerError,
       NoClient | NoPlayer(_) | GameBeingDestroyed(_)
         => Status::NotFound,
-      OE::PieceHeld | OE::PieceImmoveable |
-      OE::OverlappingOccultation | OE::Occultation |
-      OE::BadPieceStateForOperation
-        => Status::Conflict,
-      InvalidZCoord | BadOperation | BadJSON(_)
+      BadJSON(_) | BadLoose
         => Status::BadRequest,
     }
   }
 }
 
-impl<'r> Responder<'r> for OnlineErrorResponse {
+impl<'r> Responder<'r> for FatalErrorResponse {
   #[throws(Status)]
   fn respond_to(self, req: &Request) -> Response<'r> {
     let msg = format!("Online-layer error\n{:?}\n{}\n", self, self);
@@ -97,7 +102,7 @@ impl<'r> Responder<'r> for OnlineErrorResponse {
   }
 }
 
-#[throws(OE)]
+#[throws(Fatal)]
 fn api_piece_op<O: op::Complex>(form: Json<ApiPiece<O>>)
                    -> impl response::Responder<'static> {
 //  thread::sleep(Duration::from_millis(2000));
@@ -123,10 +128,9 @@ fn api_piece_op<O: op::Complex>(form: Json<ApiPiece<O>>)
   let piece = vpiece_decode(gs, player, gpl, form.piece);
   if_let!{ Some(piece) = piece; else return Ok(()) }
   let was_held = gs.pieces.get(piece).as_ref().map(|gpc| gpc.held);
-  use ApiPieceOpError::*;
 
   match (||{
-    let ipc = ipieces.get(piece).ok_or(POE::PieceGone)?;
+    let ipc = ipieces.get(piece).ok_or(Ia::PieceGone)?;
     let gpc = gs.pieces.byid_mut(piece)?;
 
     let q_gen = form.gen;
@@ -136,7 +140,10 @@ fn api_piece_op<O: op::Complex>(form: Json<ApiPiece<O>>)
 
     debug!("client={:?} pc.lastclient={:?} pc.gen_before={:?} pc.gen={:?} q_gen={:?} u_gen={:?}", &client, &gpc.lastclient, &gpc.gen_before_lastclient, &gpc.gen, &q_gen, &u_gen);
 
-    if u_gen > q_gen { throw!(PieceOpError::Conflict) }
+    let loose_conflict = if u_gen <= q_gen { None } else {
+      if ! form.loose { throw!(Inapplicable::Conflict); }
+      Some(form.op.conflict_loose_check(&gpc, client)?)
+    };
     trace_dbg!("form.op", player, piece, &form.op, &gpc);
     form.op.check_held(gpc,player)?;
     let update =
@@ -146,28 +153,33 @@ fn api_piece_op<O: op::Complex>(form: Json<ApiPiece<O>>)
         ig: &iad.gref,
         to_recalculate: &mut to_recalculate,
       })?;
-    Ok::<_,ApiPieceOpError>(update)
+    Ok::<_,ApiPieceOpError>((update, loose_conflict))
   })() {
-    Err(ReportViaUpdate(poe)) => {
+    Err(APOE::Inapplicable(poe)) => {
       PrepareUpdatesBuffer::piece_report_error(
         &mut ig, poe,
         piece, vec![], POEPP::Unprocessed, client, form.cseq,
       )?;
       debug!("api_piece_op Err(RVU): {:?}", &form);
     },
-    Err(PartiallyProcessed(poe, logents)) => {
+    Err(APOE::PartiallyProcessed(poe, logents)) => {
       PrepareUpdatesBuffer::piece_report_error(
         &mut ig, poe,
         piece, logents, POEPP::Partially, client, form.cseq,
       )?;
       debug!("api_piece_op Err(PP): {:?}", &form);
     },
-    Err(ReportViaResponse(err)) => {
+    Err(APOE::Fatal(err)) => {
       warn!("api_piece_op ERROR {:?}: {:?}", &form, &err);
       Err(err)?;
     },
-    Ok((PieceUpdate { wrc, log, ops }, unprepared)) => {
-      let by_client = Some((wrc, client, form.cseq));
+    Ok(((PieceUpdate { wrc, log, ops }, unprepared), loose_conflict)) => {
+      let by_client =
+        if let Some(ContinueDespiteConflict) = loose_conflict {
+          None
+        } else {
+          Some((wrc, client, form.cseq))
+        };
       let mut buf = PrepareUpdatesBuffer::new(g,
                                               Some(1 + log.len()));
 
@@ -199,7 +211,7 @@ fn api_piece_op<O: op::Complex>(form: Json<ApiPiece<O>>)
     prepub.finish();
   }
 
-      Ok::<(),OE>(())
+      Ok::<(),Fatal>(())
     })();
 
     let g = &mut *ig;
@@ -230,7 +242,7 @@ macro_rules! api_route_core {
     $formdef
 
     #[post($path, format="json", data="<form>")]
-    #[throws(OER)]
+    #[throws(FER)]
     fn $fn(form: Json<ApiPiece<$form>>)
            -> impl response::Responder<'static> {
       api_piece_op(form)?
@@ -275,6 +287,14 @@ api_route!{
   }
 
   impl op::Core as {
+    #[throws(ApiPieceOpError)]
+    fn conflict_loose_check(&self, gpc: &GPiece, client: ClientId)
+                            -> ContinueDespiteConflict {
+      if gpc.occult.is_active()             { throw!(Ia::Occultation    ) }
+      if gpc.moveable != PieceMoveable::Yes { throw!(Ia::PieceImmoveable) }
+      if gpc.last_released != client        { throw!(Ia::PieceHeld      ) }
+      ContinueDespiteConflict
+    }
   }
 
   impl op::Simple as {
@@ -289,7 +309,7 @@ api_route!{
         "grasped"
       )?;
 
-      if gpc.held.is_some() { throw!(OnlineError::PieceHeld) }
+      if gpc.held.is_some() { throw!(Ia::PieceHeld) }
       gpc.held = Some(player);
     
       let update = PieceUpdateOp::ModifyQuiet(());
@@ -306,7 +326,7 @@ api_route!{
   }
 
   impl op::Core as {
-    #[throws(OnlineError)]
+    #[throws(Ia)]
     fn check_held(&self, _pc: &GPiece, _player: PlayerId) { }
   }
 
@@ -322,11 +342,12 @@ api_route!{
 
       let gpl = players.byid_mut(player)?;
       let pri = piece_pri(ioccults, &gs.occults, player, gpl, piece, gpc, ipc)
-        .ok_or(POE::PieceGone)?;
+        .ok_or(Ia::PieceGone)?;
 
       let pcs = pri.describe(ioccults,&gs.occults, gpc, ipc);
 
       gpc.held = Some(player);
+      gpc.last_released = default();
 
       let update = PieceUpdateOp::Modify(());
 
@@ -388,7 +409,7 @@ api_route!{
 
       then {
         let z = gs.max_z.z.clone_mut().increment().map_err(
-          |e| APOE::ReportViaResponse(IE::from(e).into()))?;
+          |e| APOE::Fatal(IE::from(e).into()))?;
         Some(ZLevel { z, zg: gs.gen })
       }
       else { None }
@@ -401,10 +422,11 @@ api_route!{
       ioccults,&gs.occults,gpl,gpc,ipc,
       "released"
     )?;
-    let who_by = who_by.ok_or(POE::PieceGone)?;
+    let who_by = who_by.ok_or(Ia::PieceGone)?;
 
-    if gpc.held != Some(player) { throw!(OnlineError::PieceHeld) }
+    if gpc.held != Some(player) { throw!(Ia::PieceNotHeld) }
     gpc.held = None;
+    gpc.last_released = a.client;
 
     let wrc = if let Some(zlevel) = new_z {
       gpc.zlevel = zlevel;
@@ -433,7 +455,7 @@ api_route!{
         to_recalculate,
         piece,
         vanilla,
-      ).map_err(|e| OnlineError::from(e))?;
+      ).map_err(|e| Fatal::from(e))?;
 
     update
   }
@@ -450,7 +472,9 @@ api_route!{
   fn op(&self, a: ApiPieceOpArgs) -> PieceUpdate {
     let ApiPieceOpArgs { gs,piece, .. } = a;
     let gpc = gs.pieces.byid_mut(piece).unwrap();
-    if gpc.occult.is_active() { throw!(OE::Occultation) }
+    if gpc.occult.is_active() {
+      if self.z >= gpc.zlevel.z { throw!(Ia::Occultation) }
+    }
     gpc.zlevel = ZLevel { z: self.z.clone(), zg: gs.gen };
     let update = PieceUpdateOp::SetZLevel(());
     (WhatResponseToClientOp::Predictable,
@@ -463,18 +487,18 @@ api_route!{
   struct ApiPieceMove(Pos);
 
   impl op::Core as {
-    #[throws(OnlineError)]
+    #[throws(Ia)]
     fn check_held(&self, gpc: &GPiece, player: PlayerId) {
       // This will ensure that occultations are (in general) properly
       // updated, because the player will (have to) release the thing
       // again
-      if gpc.held != Some(player) { throw!(OnlineError::PieceHeld) }
-      if gpc.occult.is_active() { throw!(OE::Occultation) }
+      if gpc.held != Some(player) { throw!(Ia::PieceNotHeld) }
+      if gpc.occult.is_active() { throw!(Ia::Occultation) }
       if matches_doesnot!(
         gpc.moveable(),
         = PieceMoveable::No,
         ! PieceMoveable::Yes | PieceMoveable::IfWresting,
-      ) { throw!(OE::PieceImmoveable) }
+      ) { throw!(Ia::PieceImmoveable) }
     }
   }
 
@@ -489,7 +513,7 @@ api_route!{
         Err(pote) => {
           gpc.pos = pote.clamped;
           throw!(ApiPieceOpError::PartiallyProcessed(
-            PieceOpError::PosOffTable,
+            Inapplicable::PosOffTable,
             logents,
           ));
         }
@@ -511,7 +535,7 @@ api_route!{
     let ApiPieceOpArgs { gs,ioccults,player,piece,ipc, .. } = a;
     let gpc = gs.pieces.byid_mut(piece).unwrap();
     if ! gpc.rotateable() || gpc.occult.is_active() {
-      throw!(POE::PieceUnrotateable)
+      throw!(Ia::PieceUnrotateable)
     }
     let gpl = gs.players.byid_mut(player).unwrap();
     let logents = log_did_to_piece(
@@ -533,17 +557,22 @@ api_route!{
   #[throws(ApiPieceOpError)]
   fn op(&self, a: ApiPieceOpArgs) -> PieceUpdate {
     let ApiPieceOpArgs { gs,ioccults,player,piece,ipc, .. } = a;
+    let ops = PUOs_Simple_Modify;
+    let new_z = piece_make_heavy(&gs.pieces, piece)?;
     let gpc = gs.pieces.byid_mut(piece).unwrap();
     let gpl = gs.players.byid_mut(player).unwrap();
-    let logents = log_did_to_piece(
+    let log = log_did_to_piece(
       ioccults,&gs.occults,gpl,gpc,ipc,
       if gpc.pinned { "pinned" } else { "unpinned" },
     )?;
     gpc.forbid_involved_in_occultation()?;
     gpc.pinned = self.0;
-    let update = PieceUpdateOp::Modify(());
-    (WhatResponseToClientOp::Predictable,
-     update, logents).into()
+    gpc.zlevel.z = new_z;
+    PieceUpdate {
+      wrc: WhatResponseToClientOp::Predictable,
+      ops: ops.into(),
+      log,
+    }
   }
 }
 
@@ -566,12 +595,12 @@ api_route!{
                           player, gs.players.byid_mut(player)?,
                           piece, gs.pieces.byid(piece)?,
                           ipc)
-        .ok_or(POE::PieceGone)?;
+        .ok_or(Ia::PieceGone)?;
       let y = {
         use PriOG::*;
         match pri.occulted {
           Visible(y) => y,
-          Occulted | Displaced(..) => throw!(OE::BadOperation),
+          Occulted | Displaced(..) => throw!(Ia::Occultation),
         }
       };
 
